#ifndef UU_GETTIME_H
#define UU_GETTIME_H

#include "ulib/UUID.h"

void uu_gettime_init(pUCXT);
U64 gt_100ns64(pUCXT);

extern void (*myU2time)(pTHX_ UV ret[2]);

#endif
/* ex:set ts=2 sw=2 itab=spaces: */
